# frozen_string_literal: true

# checker Api::Dmo::HermesController avant d'ajouter des methodes
module Hermes
  module Api
    module Dmo
      class HousingApplicationsController < Hermes::Api::Dmo::ApplicationController
              require Rails.root.join('lib', 'core_extentions', 'parameters', 'handle_sort').to_s

        VALID_OPERATORS = %w[
          $in
          $nin
          $ne
          $gt
          $gte
          $lt
          $lte
          $regex
        ]

        ACCEPTABLE_PARAMS = %w[
          ids
          housing_request_id
          housing_offer_id
          epci
          max_date
          applicated_at
          status
          status_updated_at
          visit_conducted_date
          visit_status
          visit_status_updated_at
          lessor_event_informations
          last_status_update_date
          rank
          profile_score
          eligible_position
          applicated_position
          is_active
          als_id
          date_event
          cotations
          business_cotation
          applicant_used_for_siret_firstname
          applicant_used_for_siret_lastname
          applicant_used_for_siret_fullname
          applicant_used_for_siret_email
          applicant_used_for_siret_birth_date
        ]

        REGEX_PARAMS = %w[
        ]

        MUTATE_PARAMS = %i[
          housing_request_id
          housing_offer_id
          applicant_id
          nud
          birth_date
          epci
          max_date
          status
          visit_status
          visit_conducted_date
          lessor_event_informations
          last_status_update_date
          rank
          date_event
        ]
        
        def mutate_params
          MUTATE_PARAMS
        end 
        
        def index
          acceptable_params *ACCEPTABLE_PARAMS
          regex_params *REGEX_PARAMS
          valid_operators *VALID_OPERATORS

          per_page = params[:per_page]&.to_i || 15
          page = params[:page]&.to_i || 1
          options = {}

          options[:join_housing_offer]= true if params[:join_housing_offer]

          paginate(
            index_aggregation,
            each_serializer: Hermes::HousingApplicationLightSerializer,
            **options
          )
        end

        def show
          @options[:join_housing_offer]= true if params[:join_housing_offer]
          super
        end

        def update_status
          return render_json_error(:unprocessable_entity, :missing_status) unless params["data"]["attributes"]["status"]
          return render_json_error(:unprocessable_entity, :missing_unique_number) unless params["data"]["attributes"]["unique_number"]
          return render_json_error(:unprocessable_entity, :missing_external_ref) unless params["data"]["attributes"]["external_ref"]

          hr = Hermes::HousingRequest.find_by(
            is_active: true, 
            :request_status.nin => ['ANNULEE', 'SATISFAITE'], 
            unique_numbers: params["data"]["attributes"]["unique_number"]
          )
          return render_json_error(:unprocessable_entity, :housing_request_not_found) unless hr

          @instance = hr.active_application
          return render_json_error(:unprocessable_entity, :housing_application_not_found) unless @instance
          return render_json_error(:unprocessable_entity, :external_ref_not_matching_application) unless @instance.housing_offer.external_ref == params["data"]["attributes"]["external_ref"]

          unless @instance.reload && @instance.writable_by?(@user)
            Hermes::LogJob.add_to_queue(level: 'error', housing_application_id: @instance.id.to_s, code: "UPDATE_STATUS_ERR", action: "update_status -- User not writer", data: params["data"]["attributes"].as_json );
            return render_json_error(:unprocessable_entity, :not_writers)
          end
          
          old_status = @instance.status&.to_s
          new_status = params["data"]["attributes"]["status"]&.to_s

          list_old_status = Hermes::Statuses::HousingApplication.possible_status.values.flatten;
          mapping = {
            "INSCRIPTION CAL" => list_old_status,
            "RETENUE EN CAL SOUS RESERVE" => list_old_status,
            "RETENUE EN CAL" => list_old_status,
            "RETENUE EN CAL SOUS CONDITIONS" => list_old_status,
            "REFUSEE EN CAL" => list_old_status,
            "DESISTEMENT" => list_old_status,
            "VISITE PROPOSEE" => list_old_status,
            "VISITE EFFECTUEE" => list_old_status,
            "LOGE" => list_old_status
          }

          not_valid_change = new_status && old_status && mapping[new_status].present? && mapping[new_status].include?(old_status)
          unless not_valid_change
            Hermes::LogJob.add_to_queue(level: 'error', housing_application_id: @instance.id.to_s, code: "UPDATE_STATUS_ERR", action: "update_status -- cannot change #{old_status} to #{new_status}", data: params["data"]["attributes"].as_json )
            return render_json_error(:unprocessable_entity, :cannot_change_to_status_from_current_status)
          end

          @instance.force_update_status_task_for = 'status'
          if Hermes::Statuses::HousingApplication.possible_status[:visit_status].include?(params["data"]["attributes"]["status"])
            params["data"]["attributes"]["visit_status"] = params["data"]["attributes"]["status"]
            params["data"]["attributes"].delete("status")
            @instance.force_update_status_task_for = 'visit_status'
          end
          
          params["data"]["attributes"].delete("unique_number")
          params["data"]["attributes"].delete("external_ref")
          check_params_and_update
        end

        def attribution
          # https://www.wrike.com/open.htm?id=461622137
          @offer = Hermes::HousingOffer.find mutate_valid_params[:housing_offer_id]
          return render_json_error(:not_found, :object_not_found) unless @offer
          return render_json_error(:unprocessable_entity, :not_requalified) unless @offer.requalification

          if applicant_id = mutate_valid_params[:applicant_id]
            @applicant = Hermes::Applicant.find applicant_id
          elsif uniq_numb = mutate_valid_params[:nud]
            birth_date = mutate_valid_params[:birth_date]
            @applicant = Hermes::Applicant.find_or_create_by_nud(current_user, uniq_numb, birth_date)
          end

          if @applicant.errors.present?
            return render_json_error(:unprocessable_entity, :invalid_birth_date) if @applicant.errors.messages[:birth_date]
            return render_json_error(:not_found, :invalid_nud) if @applicant.errors.messages[:nud]
          end

          @applicant.housing_requests.each do |hr|
            hr.housing_applications.each do |ha|
              ha.update(status: 'ANNULEE', is_active: false)
            end # (a) si le demandeur a une candidature en cours elle est annulée
            hr.update(request_status: 'COMPLETE') # (b) passer les demandes en logé
          end

          @new_request = Hermes::HousingRequest.create(main_applicant: @applicant)
          @application = Hermes::HousingApplication.create(
            housing_offer: @offer,
            housing_request: @new_request,
            applicated_at: DateTime.current
          )
          @offer.update(lessor_status: 'LOUE', offer_status: "RETIRE PAR LE BAILLEUR") # (b) et l'offre en attribuée + (c) l'offre est comptabilisé dans les objectifs "attribués"
          @application.reload

          render(jsonapi: @application, serializer: @serializer, status: :created)
        end
      end
    end
  end
end
