# frozen_string_literal: true

module Hermes
  class EntityValidation
    include Hermes::Concerns::CommonFieldsConcern
    field :entity_id, type: BSON::ObjectId
    field :applicant_id, type: BSON::ObjectId
    field :identification_number, type: String
    field :civility, type: String
    field :first_name, type: String
    field :last_name, type: String
    field :birth_date, type: Date
    field :validation, type: Boolean
    field :contract_start_date, type: Date
    field :contract_end_date, type: Date
    field :active, type: Boolean, default: false

    belongs_to :entity, index: true, class_name: 'Hermes::Entity'
    belongs_to :applicant, index: true, class_name: 'Hermes::Applicant'

    validates_presence_of :entity_id, :applicant_id, :identification_number, :contract_start_date

    before_save :before_save_actions
    after_save :after_save_actions
    after_validation :after_validation_actions

    owner_field :user_id

    common_indexes *%i[
      identification_number
      civility
      first_name
      last_name
      birth_date
      validation
      contract_start_date
      contract_end_date
      active
    ]

    def handle_contract_end_date
      if contract_end_date && contract_end_date_changed?
        close_contract
      end
    end

    def handle_active
      self.active = Date.today >= contract_start_date
      if [contract_start_date, contract_end_date].all?(&:present?)
        self.active = Date.today.between?(contract_start_date, contract_end_date)
      end
      true # if a method return false before a save then the save fail
    end

    def handle_validation_value
      if self.validation.nil?
        self.validation = entity.auto_validation
      end
    end

    def fill_identification_fields
      unless [civility, first_name, last_name, birth_date].any?(&:present?)
        self.assign_attributes(
          civility: applicant.civility,
          first_name: applicant.first_name,
          last_name: applicant.last_name,
          birth_date: applicant.birth_date
        )
      end
    end

    def close_contract
      CloseContractJob.set(wait_until: contract_end_date.to_time).perform_later(id.to_s)
    end

    private

    def before_save_actions
      handle_active
      update_rights
      fill_identification_fields
      handle_contract_end_date
    end

    def after_validation_actions
      handle_validation_value
    end
    
    def after_save_actions
    end

    def update_rights
      if self.user_id.nil?
        self.user_id = entity.user_id
      end
    end
  end
end
