require 'rails_helper'

module Hermes
  require [Hermes::Engine.root, %w[lib hermes tools specs]].flatten.join('/')

  MUTATE_PARAMS = {
    consent_fields: 'Array',
    delegation: 'Boolean',
    housing_request_id: ['housing_request1', 'housing_request2'],
    entity_form_level_id: 'BSON::ObjectId',
    level: 'Integer'
  }

  ACCEPTABLE_PARAMS = %w[
    entity_id
    housing_request_id
    applicant_id
    consent_fields
    level
    delegation
    delegation_signed
    created_at
    updated_at
  ]

  describe Api::Dmo::HousingRequestEntityFormsController, :type => :api do
    routes { Hermes::Engine.routes }
    include Devise::Test::ControllerHelpers
    include JwtAuthHelper

    check_mutable_params(
      model: Hermes::HousingRequestEntityForm, 
      mutate_params: MUTATE_PARAMS, 
      factory: :entity_form,
      other_factory:[
        {factory: :active_request, id: 'housing_request1'},
        {factory: :active_request, id: 'housing_request2'},
      ],
      min_params: {
        housing_request_id: 'housing_request1',
      }
    )

    let!(:admin) { create(:user, roles: ['Administrator']) }
    let!(:entreprise) { create(:user, auth_roles: ['entreprise']) }
    let!(:user) { create(:user) }
    
    let!(:housing_request) { create :active_request }
    let!(:entity) { create :entity }
    let!(:user_rights) { create(:user_right, entity_id: entity.id, user_id: entreprise.id, right: 'administrator', type: 'entity') }
    let!(:entity_form_level) { create :entity_form_level, entity_id: entity.id, level: 2, delegation: true }

    let(:entity_form) { create :entity_form, entity_form_level_id: entity_form_level.id, consent_fields: ['firstname', 'lastname'] }
    let(:params) do
      {
        attributes: {
          consent_fields: ['field_1', 'field_2'],
          entity_form_level_id: entity_form_level.id,
          housing_request_id: housing_request.id
        }
      }
    end

    before do
      authenticated_header(request, admin)
    end

    describe 'GET index' do
      let!(:entity_forms) { create_list :entity_form, 5 }
      before { get 'index' }
      it 'succeeds' do
        expect(response).to have_http_status 200
      end
      it 'returns a valid json' do
        expect(json['data'].length).to eq 5
        expect(json['meta']['pagination']).to be_present
        expect(json['meta']['pagination']['total_objects']).to eq 5
      end
    end

    describe 'GET show' do
      before { get 'show', { id: entity_form.id } }
      it 'succeeds' do
        expect(response).to have_http_status 200
      end
      it 'returns a valid json' do
        expect(json['data']['id']).to eq entity_form.id.to_s
        expect(json['data']['attributes']['entity_id']).to eq entity_form_level.entity_id.to_s
      end
    end

    describe 'POST create' do
      subject { post 'create', { data: params } }
      it 'succeeds' do
        subject
        expect(response).to have_http_status 201
      end
      it 'creates an entity_form' do
        expect { subject }.to change { Hermes::HousingRequestEntityForm.count }.by 1
      end
      it 'returns a valid json' do
        subject
        expect(json['data']['attributes']['consent_fields']).to eq ['field_1', 'field_2']
        expect(json['data']['attributes']['entity_id']).to eq entity_form_level.entity_id.to_s
        expect(json['data']['attributes']['delegation']).to eq entity_form_level.delegation
      end

      it 'change consent_level on housing_request' do
        subject
        expect(housing_request.reload.consent_level).to eq 2
      end

      context 'temporary without entity_form_level' do
        let(:random_entity) { create :entity }
        let(:params) do
          {
            attributes: {
              consent_fields: ['field_1', 'field_2'],
              level: 3,
              housing_request_id: housing_request.id
            }
          }
        end
        it 'succeeds' do
          subject
          expect(response).to have_http_status 201
        end
        it 'creates an entity_form' do
          expect { subject }.to change { Hermes::HousingRequestEntityForm.count }.by 1
        end
        it 'returns a valid json' do
          subject
          expect(json['data']['attributes']['level']).to eq 3
        end
      end
    end

    describe 'PATCH update' do
      subject { patch 'update', { id: entity_form.id, data: params } }
      it 'succeeds' do
        subject
        expect(response).to have_http_status 200
      end
      it 'updates the entity_form' do
        expect { subject }.to change { entity_form.reload.consent_fields }.from(['firstname', 'lastname']).to ['field_1', 'field_2']
      end
      it 'returns a valid json' do
        subject
        expect(json['data']['id']).to eq entity_form.id.to_s
        expect(json['data']['attributes']['entity_id']).to eq entity_form_level.entity_id.to_s
      end
    end

    describe 'DELETE destroy' do
      let!(:owned_entity_form) { create :entity_form, user_id: entreprise.id }
      subject { delete 'destroy', { entity_id: entity.id, id: owned_entity_form.id } }
      before do
        authenticated_header(request, entreprise)
      end
      it 'succeeds' do
        subject
        expect(response).to have_http_status 204
      end
      it 'destroys the entity_form' do
        expect { subject }.to change { Hermes::HousingRequestEntityForm.count }.by -1
      end
    end

    describe 'GET show_request' do
      context 'when entreprise' do
        before do
          authenticated_header(request, entreprise)
          get 'show_request', { id: entity_form.id }
        end
        it 'succeeds' do
          expect(response).to have_http_status 200
        end
        it 'returns the list of consent_fields' do
          expect(json).to eq ['firstname', 'lastname']
        end
      end
      context 'when other role' do
        before do
          authenticated_header(request, admin)
          get 'show_request', { id: entity_form.id }
        end
        it 'returns unauthorized error' do
          expect(response).to have_http_status 401
        end
      end
    end
  end

  require_relative '../../swagger_helper'

  describe Hermes::Api::Dmo::HousingRequestEntityFormsController, type: :request, swagger_doc: 'api/swagger/hermes/swagger_hermes.json' do
    TAGS_API_DMO_HOUSING_REQUEST_ENTITY_FORMS = 'API_DMO_HOUSING_REQUEST_ENTITY_FORMS'

    include JwtAuthHelper
    extend HermesSwaggerJsonParams
    include SwaggerJsonAttributes

    context 'as admin' do
      let!(:admin) { create(:user, roles: ['Administrator']) }

      path '/api/dmo/housing_request_entity_forms' do
        post 'Create housing_request_entity_form' do
          tags TAGS_API_DMO_HOUSING_REQUEST_ENTITY_FORMS
          produces 'application/json'
          consumes 'application/json'
          security [Bearer: {}]
          json_params = {}
          MUTATE_PARAMS.each do |key, val|
            json_params[key] = { type: nice_type(val) }
          end
          parameter name: :params, in: :body, schema: jsonify_params(json_params)

          response '201', 'housing_request_entity_form created' do
            let(:Authorization) { "Bearer #{token_for(admin)}" }
            let(:params) do
              jsonapi_attributes({
                housing_request_id: 'string',
                entity_form_level_id: 'string'
              })
            end
            include_context 'with integration test'
            include_context 'valid jsonapi'
            include_context 'not containings errors'

            after do |_example|
              json = JSON.parse(response.body)
              expect(json['data']).to be_a Hash
              expect(json['data']['id']).to_not be nil
              expect(json['data']['type']).to eq 'housing_request_entity_forms'
            end
          end
        end
      end

      path '/api/dmo/housing_request_entity_forms' do
        get 'Retrieves all housing_request_entity_forms' do
          tags TAGS_API_DMO_HOUSING_REQUEST_ENTITY_FORMS
          produces 'application/json'
          parameter name: 'ids[]', in: :query, type: :array, items: { type: :string }, collectionFormat: :multi, required: false
          ACCEPTABLE_PARAMS.each do |acceptable_param|
            type = ''
            if MUTATE_PARAMS.include?(acceptable_param)
              type = nice_type(MUTATE_PARAMS[acceptable_param])
            end
            parameter name: acceptable_param.to_s, in: :query, type: type, required: false
          end

          security [Bearer: {}]
          paginator

          response '200', 'housing_request_entity_forms found' do
            let(:Authorization) { "Bearer #{token_for(admin)}" }
            let!(:entity_forms) { create_list(:entity_form, 5) }

            include_context 'with integration test'
            include_context 'valid jsonapi collection' do
              let(:expected_collection) { entity_forms }
            end
          end
        end
      end

      path '/api/dmo/housing_request_entity_forms/{id}' do
        get 'Retrieves a single housing_request_entity_form by id' do
          tags TAGS_API_DMO_HOUSING_REQUEST_ENTITY_FORMS
          produces 'application/json'
          parameter name: :id, in: :path, type: :string
          security [Bearer: {}]

          response '200', 'housing_request_entity_form found' do
            let(:Authorization) { "Bearer #{token_for(admin)}" }
            let(:entity_form) { create(:entity_form) }
            let(:id) { entity_form.id }

            include_context 'with integration test'
            include_context 'valid jsonapi ressource' do
              let(:expected_ressource) { entity_form }
            end
          end
        end
      end

      path '/api/dmo/housing_request_entity_forms/{id}' do
        put 'edit housing_request_entity_form' do
          tags TAGS_API_DMO_HOUSING_REQUEST_ENTITY_FORMS
          produces 'application/json'
          consumes 'application/json'
          parameter name: :id, in: :path, type: :string
          security [Bearer: {}]
          json_params = {}
          MUTATE_PARAMS.each do |key, val|
            json_params[key] = {type: nice_type(val)}
          end
          parameter name: :params, in: :body, schema: jsonify_params(json_params)

          response '200', 'housing_request_entity_form updated' do
            let(:Authorization) { "Bearer #{token_for(admin)}" }
            let(:entity_form) { create(:entity_form) }
            let(:id) { entity_form.id }
            let(:params) do
              jsonapi_attributes({
                consent_fields: []
              })
            end
            include_context 'with integration test'
            include_context 'valid jsonapi'
            include_context 'not containings errors'

            after do |_example|
              json = JSON.parse(response.body)
              expect(json['data']).to be_a Hash
              expect(json['data']['id']).to eq entity_form.id.to_s
              expect(json['data']['type']).to eq 'housing_request_entity_forms'
            end
          end
        end
      end

      path '/api/dmo/housing_request_entity_forms/{id}' do
        delete 'Delete a single housing_request_entity_form by id' do
          tags TAGS_API_DMO_HOUSING_REQUEST_ENTITY_FORMS
          produces 'application/json'
          parameter name: :id, in: :path, type: :string
          security [Bearer: {}]

          response '204', 'housing_request_entity_form deleted' do
            let(:Authorization) {"Bearer #{token_for(admin)}"}
            let(:id) { create(:entity_form, user_id: admin.id).id }
            run_test!
          end
        end
      end
    end

    context 'as entreprise' do
      let!(:entreprise) { create(:user, roles: ['entreprise']) }
      let!(:entity) { create :entity, readers: [entreprise.id.to_s] }
      let!(:entity_form_level) { create :entity_form_level, entity_id: entity.id }

      path '/api/dmo/housing_request_entity_forms/{id}/show_request' do
        get 'Displays the consent_fields list for a single housing_request_entity_form' do
          tags TAGS_API_DMO_HOUSING_REQUEST_ENTITY_FORMS
          produces 'application/json'
          parameter name: :id, in: :path, type: :string
          security [Bearer: {}]

          response '200', 'housing_request_entity_form consent_fields displayed' do
            let(:Authorization) { "Bearer #{token_for(entreprise)}" }
            let(:entity_form) { create :entity_form, entity_form_level_id: entity_form_level.id, consent_fields: ['firstname', 'lastname'] }
            let(:id) { entity_form.id }
            run_test!
            after do
              json = JSON.parse(response.body)
              expect(json).to be_a Array
              expect(json).to_not be nil
              expect(json).to eq ['firstname', 'lastname']
            end
          end
        end
      end
    end
  end
end