require 'rails_helper'
require 'vcr_config'

module Hermes
  require [Hermes::Engine.root, %w[lib hermes tools specs]].flatten.join('/')
  include Rack::Test::Methods

  describe Api::Dmo::RequiredDocumentsController, type: :api do
    routes {Hermes::Engine.routes}
    include Devise::Test::ControllerHelpers
    include JwtAuthHelper

    let!(:responsible) {create(:user, roles: ['Administrator'])}
    let!(:applicant) {create(:applicant, user: responsible, nationality: "1")}
    let!(:housing_request1) {create(:housing_request, main_applicant_id: applicant.id.to_s)}
    let(:required_document_id) {housing_request1.housing_request_folder.required_document_ids[0]}

    describe 'PUT /required_documents/:id' do
      shared_examples 'can update required documents' do
        let(:valid_params) do
          {
            id: required_document_id.to_s,
            data: {attributes: {status: "A contrôler"}}
          }
        end
        before :each do
          authenticated_header(request, user)
          put 'update', valid_params
        end

        it 'returns status code 200' do
          expect(response.status).to eq(200)
        end
        it 'updated form' do
          expect(json['data']['id']).to eq(required_document_id.to_s)
          expect(json['data']['attributes']['status']).to eq("A contrôler")
          expect(json['data']['attributes']['status_history'].first['title']).to eq("A contrôler")
        end
      end

      context 'as collaborateur' do
        it_behaves_like 'can update required documents' do
          let(:user) { create(:user, auth_roles: ['collaborateur offre logement']) }
        end
      end

      context 'as superadministrateur' do
        it_behaves_like 'can update required documents' do
          let(:user) { create(:user, auth_roles: ['superadministrateur']) }
        end
      end
    end


    describe 'GET /required_documents/:id/historical_status' do
      let!(:status) {create(:status, object_id: required_document_id)}
      let!(:collaborator) {create(:user, roles: ['collaborateur offre logement'])}
      before :each do
        authenticated_header(request, collaborator)
        get 'historical_status', {id: required_document_id}
      end

      it 'returns status code 200' do
        resp = JSON.parse(response.body)
        expect(resp['code']).to eq(200)
      end
    end
  end


  require_relative '../../swagger_helper'

  describe Api::Dmo::RequiredDocumentsController, type: :request, swagger_doc: 'api/swagger/hermes/swagger_hermes.json' do
    TAGS_API_DMO_REQUIRED_DOCUMENTS = 'API_DMO_REQUIRED_DOCUMENTS'
    include JwtAuthHelper
    extend HermesSwaggerJsonParams
    include SwaggerJsonAttributes
    let!(:responsible) {create(:user, first_name: 'Martin', last_name: 'Durand', email: 'martin.durand@gmail.com', roles: ['Administrator'])}
    let!(:user) {create(:user, first_name: 'Jackie', last_name: 'Mike', email: 'jackie.mike@gmail.com')}
    let!(:applicant) {create(:applicant, user: user, nationality: "1")}
    let!(:housing_request1) {create(:housing_request,user: user, main_applicant_id: applicant.id.to_s)}

  end
end
